﻿const { app, BrowserWindow, ipcMain, systemPreferences } = require("electron");
const path = require("path");
const { electron } = require("process");

let mainWindow = null;
global.sharedObject = {
  mainWindowId: null,
};

// Check and apply for device permissions: microphone, camera, screen recording
async function checkAndApplyDeviceAccessPrivilege() {
  const cameraPrivilege = systemPreferences.getMediaAccessStatus("camera");
  console.log(
    `checkAndApplyDeviceAccessPrivilege before apply cameraPrivilege: ${cameraPrivilege}`
  );
  if (cameraPrivilege !== "granted") {
    await systemPreferences.askForMediaAccess("camera");
  }

  const micPrivilege = systemPreferences.getMediaAccessStatus("microphone");
  console.log(`checkAndApplyDeviceAccessPrivilege before apply micPrivilege: ${micPrivilege}`);
  if (micPrivilege !== "granted") {
    await systemPreferences.askForMediaAccess("microphone");
  }

  const screenPrivilege = systemPreferences.getMediaAccessStatus("screen");
  console.log(
    `checkAndApplyDeviceAccessPrivilege before apply screenPrivilege: ${screenPrivilege}`
  );
}

function initAppListeners() {
  ipcMain.on("createShareScreenWindow", (event, arg) => {
    event.returnValue = 0;
  });
  ipcMain.on("destroyShareScreenWindow", (event, arg) => {
    event.returnValue = 0;
  });
}

function initApp() {
  // Create the main view
  function createWindow() {
    const windowOptions = {
      width: 1080,
      minWidth: 680,
      height: 840,
      show: false,
      title: app.getName(),
      webPreferences: {
        webSecurity: false,
        nodeIntegration: true,
        contextIsolation: false,
        preload: path.join(__dirname, "preload.js"),
        enableRemoteModule: true,
      },
    };
    mainWindow = new BrowserWindow(windowOptions);
    // load view
    mainWindow.loadFile("index.html");
    mainWindow.once("ready-to-show", () => {
      mainWindow.show();
    });
    mainWindow.webContents.openDevTools();
    global.sharedObject.mainWindowId = mainWindow.webContents.id;

    mainWindow.on("closed", () => {
      mainWindow = null;
    });
  }

  app.whenReady().then(async () => {
    await checkAndApplyDeviceAccessPrivilege();
    createWindow();
    app.on("activate", () => {
      if (mainWindow === null) {
        createWindow();
      }
    });
  });

  app.on("window-all-closed", () => {
    // if (process.platform !== "darwin") app.quit();
    app.quit();
  });
}

function main() {
  initAppListeners();
  initApp();
}

main();
