/* eslint-disable camelcase */
/**
 * VolcEngineRTC The main page of the video call
 * This example limits the maximum number of users in the room; the video data of up to four users (self and three remote users' video data) can be rendered at the same time；
 *
 * Contains the following simple functions:
 * - Create RTCVideo and RTCRoom
 * - Join the audio and video chat room
 * - Render your own video data
 * - Turn on/off the microphone
 * - Turn on/off the camera
 * - Render the video data of the remote user
 * - leave the room
 * - Destroy RTCRoom and RTCVideo
 *
 * The process of implementing a basic audio and video call is as follows:
 * 1.Create RTCVideo and RTCRoom   rtcVideo.createRTCVideo() rtcRoom.createRTCRoom();
 * 2.Join the audio and video chat room        rtcRoom.joinRoom();
 * 3.Enable local audio and video capture        rtcVideo.startAudioCapture() rtcVideo.startVideoCapture();
 * 4.Set the local video rendering view       rtcVideo.setupLocalVideo();
 * 5.After receiving the first frame of the remote user's video, set the user's video rendering view rtcVideo.setupRemoteVideo（）
 * 6.Leave the audio and video call room        rtcRoom.leaveRoom();
 * 7.Destroy RTCRoom and RTCVideo   rtcRoom.destroy() rtcVideo.destroyRTCVideo():
 
 * There are the following common considerations:
 * 1.In this example, we set the remote user's video rendering view for the remote user in the onFirstRemoteVideoFrameDecoded event. This callback indicates that the first frame of the video from the remote user is received. Of course, you can also set the remote user video rendering view in the onUserJoined:elapsed: callback
 * 2.SDK The callback is not on the main thread, UI operations need to switch threads
 * 3.After the user successfully joins the room, the SDK will call back the user information already in the room through OnUserJoined
 * 4.SDK Support to publish video streams with multiple parameters at the same time, the interface is setVideoEncoderConfig:
 * 5.When joining a room, a valid token is required. When the joining fails, the corresponding error code will be output through onError:
 * 6.Users can use roomConfig in rtcRoom.joinRoom(); to obtain performance optimization in different scenarios. This example is an audio and video call scenario
 * 7.No need to destroy the engine every time you join/exit a room. This example destroys the engine when exiting the room to show the complete usage flow
 *
 * For detailed API documentation see: https://www.volcengine.com/docs/6348/85532
 */
const config = require("../config/config");
const $ = require("./jquery-3.4.1.min.js");
const OFFVIDEOICON = "./assets/videoOffIcon.png";
const ONVIDEOICON = "./assets/videoOnIcon.png";
const OFFMICICON = "./assets/micOffIcon.png";
const ONMICICON = "./assets/micOnIcon.png";
const TEXTMAP = {
  idRoomId: "RoomID",
  idUserId: "UserID",
};
///////////////////////////////////////////////////////////////////
$("#control").hide();
$(".player").hide();
$("#room-id-text").text("");
$("#user-id-text").text("");
checkRoomIdAndUserId("idRoomId");
checkRoomIdAndUserId("idUserId");
///////////////////////////////////////////
// rtcVideo rtcRoom
let rtcVideo;
let rtcRoom;

// Note: please enter your appId
let appId = config.appId;
let token = config.token;
// Note: room id
let roomId = "";
// Note: user id
let userId = "";

let isMicOn = false; // control the mic
let isVideoOn = false; // control the video

let remoteUsers = [];
// UI Elements
let idRoomId, idUserId;
let idJoin, idLeave;
let idOpenMicrophone;
let idOpenCamera;
let idLocalVideoContainer;
let isVideoCreated = false;
let isRoomCreated = false;
// If you only want to send one audio stream,
// you can configure only one set of audio and video resolution information
let roomConfig = {
  room_profile_type: 1,
  is_auto_publish: true,
  is_auto_subscribe_audio: true,
  is_auto_subscribe_video: true,
};

/////////////////////////////////////////////////////////////////////////
// Callback
// Someone joins the audio and video conference
const userJoinedHandler = (userInfo, elapsed) => {
  let joinUserId = userInfo.uid;
  let flag = remoteUsers.includes(joinUserId);
  if (!flag && remoteUsers.length < 3) {
    remoteUsers.push(joinUserId);
    const player = $(`
    <div id="player-wrapper-${joinUserId}" class="player-wrapper re-player">
      <p class="player-name">${joinUserId}</p>
    </div>
  `);
    $("#player-list").append(player);
  }
};
// Someone leaves the audio/video conference
const userOfflineHandler = (userId, reason) => {
  remoteUsers = remoteUsers.filter((remoteUserId) => {
    return remoteUserId !== userId;
  });
  $(`#player-wrapper-${userId}`).remove();
};

// The first frame of the remote user's video is received
const firstRemoteVideoFrameHandler = (key, info) => {
  let streamIndex = key.stream_index;
  const remoteUserId = "player-wrapper-" + key.user_id;
  const container = document.getElementById(remoteUserId);
  if (streamIndex == 0 && container) {
    ///Set the remote user video rendering view
    rtcVideo.setupRemoteVideo(key.user_id, key.room_id, container, 1);
  }
};

///Received SDK danger warning callback
const warningHandler = (warn) => {};
///Receive sdk error callback
const errorHandler = (error) => {};
rtcVideo = new window.veRTCVideo();

///Initialize RTCVideo RTCRoom
createVideo();

document.getElementById("header-version").innerHTML =
  "BytePlusRTC " +
  window.veRTCVideo.getSDKVersion() +
  " DebugProcessId " +
  window.veRTCVideo.getCurrentProcessId();

//
function checkRoomIdAndUserId(name) {
  $(`#${name}`).keyup(() => {
    checkReg(name);
  });
}

function checkReg(name) {
  const inputValue = $(`#${name}`).val();
  let hasError = false;
  if (!inputValue) {
    $(`#${name}`).css("border-color", "#ff4d4f");
    $(`#${name}-text`).text(`${TEXTMAP[name]} can not be empty!`);
    hasError = true;
  } else if (!/^[0-9a-zA-Z_\-@]{1,128}$/.test(inputValue)) {
    $(`#${name}`).css("border-color", "#ff4d4f");
    $(`#${name}-text`).text(`${TEXTMAP[name]} format error!`);
    hasError = true;
  } else {
    $(`#${name}-text`).text("");
    $(`#${name}`).css("border-color", "#d9d9d9");
    hasError = false;
  }
  return hasError;
}

/////////////////////////////////////////////////////////////////////////
// Callback Function
function initVideoEventListeners() {
  ///Monitor the callback of the first frame of the video of the remote user
  rtcVideo.on("onFirstRemoteVideoFrameDecoded", firstRemoteVideoFrameHandler);
  ///Monitor for RTC warnings
  rtcVideo.on("onWarning", warningHandler);
  ///Monitor RTC error
  rtcVideo.on("onError", errorHandler);
}

function initRoomEventListener() {
  ///Monitor user joins the audio and video room callback
  rtcRoom.on("onUserJoined", userJoinedHandler);
  ///Monitor for the user to leave the audio and video room callback
  rtcRoom.on("onUserLeave", userOfflineHandler);
}

function createVideo() {
  console.log("createVideo");
  if (!rtcVideo) {
    rtcVideo = new window.veRTCVideo();
  }
  ///Initialize RTCVideo
  rtcVideo.createRTCVideo(appId, "", JSON.stringify(""));
  ///Register Video callback listener
  initVideoEventListeners();
  isVideoCreated = true;
  onBtnOpenCamera();
  onBtnOpenMicrophone();
}

function destroyRTCVideo() {
  if (rtcRoom) {
    ///Destroy the RTC engine
    rtcRoom.destroy();
    rtcRoom = null;
  }
  if (rtcVideo) {
    ///Destroy the RTC engine
    rtcVideo.destroyRTCVideo();
    rtcVideo = null;
  }
  isVideoCreated = false;
  isRoomCreated = false;
}

function onBtnJoin() {
  if (checkReg("idRoomId") || checkReg("idUserId")) {
    return;
  }
  roomId = idRoomId.value;
  userId = idUserId.value;
  $("#control").show();
  $(".player").show();
  $("#pannel").hide();
  $("#header-version").text(`${roomId}`);
  $("#local-player-name").text(`${userId}`);

  if (!isVideoCreated || rtcVideo == null) {
    createVideo();
  }
  if (!isRoomCreated) {
    //Create rtcRoom
    rtcRoom = rtcVideo.createRTCRoom(roomId);
    isRoomCreated = true;
    //Register room callback listener
    initRoomEventListener();
  }
  ///Join the audio and video room
  let ret = rtcRoom.joinRoom(token, { uid: userId }, roomConfig);
  // Set up local video rendering view
  rtcVideo.setupLocalVideo(idLocalVideoContainer, roomId);
  console.log("join room ret : " + ret);
}

function onBtnLeave() {
  ///leave the audio and video room
  rtcRoom.leaveRoom();
  //destroy room
  rtcRoom.destroy();
  isRoomCreated = false;
  $("#control").hide();
  $(".player").hide();
  $("#pannel").show();
  rtcVideo.removeLocalVideo(idRoomId.value);
  remoteUsers.forEach((remoteUserId) => {
    $(`#player-wrapper-${remoteUserId}`).remove();
  });
  remoteUsers = [];
  document.getElementById("header-version").innerHTML =
    "VolcEngingRTC " +
    window.veRTCVideo.getSDKVersion() +
    " DebugProcessId " +
    window.veRTCVideo.getCurrentProcessId();
}

function onBtnOpenMicrophone() {
  isMicOn = !isMicOn;
  if (isMicOn) {
    ///Enable local audio collection
    rtcVideo.startAudioCapture();
    $(`#idOpenMicrophone img`).attr("src", `${ONMICICON}`);
  } else {
    ///Close local audio capture
    rtcVideo.stopAudioCapture();
    $(`#idOpenMicrophone img`).attr("src", `${OFFMICICON}`);
  }
}

function onBtnOpenCamera() {
  isVideoOn = !isVideoOn;
  if (isVideoOn) {
    ///Start local video capture
    rtcVideo.startVideoCapture();
    $(`#idOpenCamera img`).attr("src", `${ONVIDEOICON}`);
  } else {
    ///Close local video capture
    rtcVideo.stopVideoCapture();
    $(`#idOpenCamera img`).attr("src", `${OFFVIDEOICON}`);
  }
}

window.onload = () => {
  idRoomId = document.getElementById("idRoomId");
  idUserId = document.getElementById("idUserId");
  idJoin = document.getElementById("idJoin");
  idLeave = document.getElementById("idLeave");
  idOpenMicrophone = document.getElementById("idOpenMicrophone");
  idOpenCamera = document.getElementById("idOpenCamera");
  idLocalVideoContainer = document.getElementById("idLocalVideoContainer");

  idJoin.addEventListener("click", onBtnJoin);
  idLeave.addEventListener("click", onBtnLeave);
  idOpenMicrophone.addEventListener("click", onBtnOpenMicrophone);
  idOpenCamera.addEventListener("click", onBtnOpenCamera);
};
